﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win.UltraWinExplorerBar;
using Infragistics.Win.UltraWinGrid;

namespace WG_OutlookStyle
{
    public partial class Form1 : Form
    {
        Random r = new Random();
        Dictionary<string, DataTable> dataSources = new Dictionary<string, DataTable>();

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SetUpItemsDataSource();
        }

        private void SetUpItemsDataSource()
        {
            foreach (var group in ultraExplorerBar1.Groups)
            {
                foreach (var item in group.Items)
                {
                    DataTable currentTable = CreateDummyTable();
                    dataSources.Add(GetDataSourceString(item), currentTable);
                }
            }
        }

        private static string GetDataSourceString(Infragistics.Win.UltraWinExplorerBar.UltraExplorerBarItem item)
        {
            return item.Text + item.Group.Text;
        }

        private DataTable CreateDummyTable()
        {
            DataTable retTable = new DataTable();

            retTable.Columns.Add("id", typeof(int)).AutoIncrement = true;
            retTable.Columns.Add("Name", typeof(string));
            retTable.Columns.Add("Duration", typeof(TimeSpan));

            int numOfActivities = r.Next(1, 5);

            for (int i = 0; i < numOfActivities; i++)
            {
                retTable.Rows.Add(new object[] { null, "Activity " + i, TimeSpan.FromHours(r.Next(1, 3)) });
            }

            return retTable;
        }

        private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            e.Layout.GroupByBox.Hidden = true;
            e.Layout.NoDataSourceMessageEnabled = Infragistics.Win.DefaultableBoolean.True;
            e.Layout.NoDataSourceMessageText = "Not data was found for this activity!";

            e.Layout.Override.ActiveAppearancesEnabled = Infragistics.Win.DefaultableBoolean.False;
            e.Layout.Override.CellClickAction = Infragistics.Win.UltraWinGrid.CellClickAction.RowSelect;
            e.Layout.Override.SelectTypeRow = Infragistics.Win.UltraWinGrid.SelectType.Single;
        }

        private void ultraExplorerBar1_ItemClick(object sender, Infragistics.Win.UltraWinExplorerBar.ItemEventArgs e)
        {
            SetGridDataSource(e.Item);
        }

        private void SetGridDataSource(Infragistics.Win.UltraWinExplorerBar.UltraExplorerBarItem item)
        {
            DataTable cachedTable = GetDataSourceItem(item);
            ultraGrid1.DataSource = cachedTable;
        }

        private DataTable GetDataSourceItem(Infragistics.Win.UltraWinExplorerBar.UltraExplorerBarItem item)
        {
            DataTable cachedTable = null;
            dataSources.TryGetValue(GetDataSourceString(item), out cachedTable);
            return cachedTable;
        }

        private void ultraGrid1_SelectionDrag(object sender, CancelEventArgs e)
        {
            if (ultraGrid1.Selected.Rows.Count == 1)
            {
                ultraGrid1.DoDragDrop(ultraGrid1.Selected.Rows[0], DragDropEffects.Move);
            }
        }

        private void ultraExplorerBar1_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(UltraGridRow)))
            {
                e.Effect = DragDropEffects.Move; 
            }
        }

        private void ultraExplorerBar1_DragDrop(object sender, DragEventArgs e)
        {
            var row = e.Data.GetData(typeof(UltraGridRow)) as UltraGridRow;

            if (row == null)
                return;

            var mouseLoc = ultraExplorerBar1.PointToClient(new Point(e.X, e.Y));

            var element = ultraExplorerBar1.UIElement.ElementFromPoint(mouseLoc);

            var item = element.GetContext(typeof(UltraExplorerBarItem),true) as UltraExplorerBarItem;

            if (item != null)
            {
                ChangeRowDataTable(row, item);
                return;
            }

            var group = element.GetContext(typeof(UltraExplorerBarGroup), true) as UltraExplorerBarGroup;

            if (group != null)
            {
                AddNewActivity(row, group);
                return;
            }

            AddNewGroup(row);
        }

        private void AddNewGroup(UltraGridRow row)
        {
            UltraExplorerBarGroup group = new UltraExplorerBarGroup();
            var lastGroup = ultraExplorerBar1.Groups[ultraExplorerBar1.Groups.Count - 1];
            DateTime lastDate;
            DateTime.TryParse(lastGroup.Text, out lastDate);
            group.Text = lastDate.AddDays(1).ToShortDateString();
            ultraExplorerBar1.Groups.Add(group);

            UltraExplorerBarItem item = AddItemToGroup(group);
            CreateDataSourceForNewItem(row, item);
        }

        private void AddNewActivity(UltraGridRow row, UltraExplorerBarGroup group)
        {
            UltraExplorerBarItem item = AddItemToGroup(group);
            CreateDataSourceForNewItem(row, item);
        }

        private void CreateDataSourceForNewItem(UltraGridRow row, UltraExplorerBarItem item)
        {
            DataTable currentTable = CreateDummyTable();
            dataSources.Add(GetDataSourceString(item), currentTable);
            var dataRow = row.ListObject as DataRowView;
            dataRow.Row.Delete();
        }

        private static UltraExplorerBarItem AddItemToGroup(UltraExplorerBarGroup group)
        {
            UltraExplorerBarItem item = new UltraExplorerBarItem();
            item.Text = String.Format("Activity {0}", group.Items.Count + 1);
            group.Items.Add(item);
            return item;
        }

        private void ChangeRowDataTable(UltraGridRow row, UltraExplorerBarItem item)
        {
            var dataRow = row.ListObject as DataRowView;
            DataTable cachedTable = GetDataSourceItem(item);
            if (dataRow.Row.Table != cachedTable)
            {
                cachedTable.Rows.Add(dataRow.Row.ItemArray);
                dataRow.Row.Delete();  
            }   
        }


    }
}
